{-|
Module      : Data.Louvain.Formats.Gexf
Description : 
Copyright   : (c) CNRS, 2017-Present
License     : AGPL + CECILL v3
Maintainer  : team@gargantext.org
Stability   : experimental
Portability : POSIX

Tools to manage GEXF Format Graphs.
-}

{-# LANGUAGE Arrows, NoMonomorphismRestriction #-}

module Data.Graph.Clustering.Louvain.IO.Gexf (readGexf)
 where

import Protolude

import Data.Text as T
import Text.Read (readMaybe)

import Text.XML.HXT.Core
-- import qualified Data.Graph as DataGraph
import qualified Data.Graph.Inductive as FGL
import System.Environment (getArgs)

data Graph = Graph
  { graphId :: Text,
    nodes :: [Text],
    edges :: [(Text, Text)] -- (Source, target)
  }
  deriving (Show, Eq)

atTag tag = deep (isElem >>> hasName tag)

parseEdges = atTag "edge" >>>
  proc e -> do
      source <- getAttrValue "source" -< e
      target <- getAttrValue "target" -< e
      returnA -< (source, target)

parseNodes = atTag "node" >>>
  proc n -> do
      nodeId <- getAttrValue "id" -< n
      returnA -< nodeId

parseGraph = atTag "graph" >>>
  proc g -> do
      graphId' <- getAttrValue "id" -< g
      nodes' <- listA parseNodes -< g
      edges' <- listA parseEdges -< g
      let graphId = T.pack graphId'
          nodes = Protolude.map T.pack nodes'
          edges = Protolude.map (\(s, t) -> (T.pack s, T.pack t)) edges'
      returnA -< Graph graphId nodes edges

getEdges = atTag "edge" >>> getAttrValue "source"

-- Get targets for a single node in a Graph
getTargets :: Text -> Graph -> [Text]
getTargets source graph = Protolude.map snd $ Protolude.filter ((==source).fst) $ edges graph

-- Convert a graph node into a Data.Graph-usable
-- getDataGraphNode :: Graph -> Text -> (Text, Text, [Text])
-- getDataGraphNode graph node = (node, node, getTargets node graph)
-- 
-- 
-- getDataGraphNode' :: Graph -> Text -> (Int, [Int])
-- getDataGraphNode' graph node = (read node, Prelude.map read (getTargets node graph))
-- 
-- -- Convert a Graph instance into a Data.Graph list of (node, nodeid, edge) tuples
-- getDataGraphNodeList :: Graph -> [(Text, Text, [Text])]
-- getDataGraphNodeList graph = map (getDataGraphNode graph) (nodes graph)
-- 
-- getDataGraphNodeList' :: Graph -> [(Int, [Int])]
-- getDataGraphNodeList' graph = map (getDataGraphNode' graph) (nodes graph)
-- 
-- --  Convert Graph structure to Data.Graph-importable tuple list
-- importGraph :: FilePath -> IO [(Int, [Int])]
-- importGraph file = do
--     graphs <- runX (readDocument [withValidate no] file >>> parseGraph)
--     let graphEdges = getDataGraphNodeList' $ head graphs
--     return graphEdges
-- 
importGraph' :: FilePath -> IO [Graph]
importGraph' file = runX (readDocument [withValidate no] file >>> parseGraph)

readGexf :: FilePath -> IO [FGL.LEdge Double]
readGexf file = do
  imported <- importGraph' file
  let mHead = Protolude.head imported
  case mHead of
    Nothing -> return []
    Just head -> do
      return $ Protolude.mapMaybe mapping $ edges head
  where
    mapping :: (Text, Text) -> Maybe (FGL.Node, FGL.Node, Double)
    mapping (a, b) =
      case (mReadA, mReadB) of
        (Nothing, _) -> Nothing
        (_, Nothing) -> Nothing
        (Just readA, Just readB) -> Just (readA :: FGL.Node, readB :: FGL.Node, 1.0)
      where
        mReadA = readMaybe (T.unpack a) :: Maybe Int
        mReadB = readMaybe (T.unpack b) :: Maybe Int


--main :: IO()
-- main = do
--     [file] <- getArgs
--     importGraph file >>= print
-- 
    -- Convert to a Data.Graph
    -- let (graph, vertexMap) = DataGraph.graphFromEdges' graphEdges
    -- Example of what to do with the Graph: Print vertices
    -- print $ map ((\ (vid, _, _) -> vid) . vertexMap) (DataGraph.vertices graph)
