{-|
Module      : Data.Louvain.Utils
Description : 
Copyright   : (c) CNRS, 2017-Present
License     : AGPL + CECILL v3
Maintainer  : alexandre.delanoe+louvain@iscpif.fr
Stability   : experimental
Portability : POSIX

Tools to manage Graphs.
-}

module Data.Graph.Clustering.Louvain.Utils
  where

import Protolude

import Data.Graph.Inductive
import Data.List (lookup, nub)
import qualified Data.Map.Strict as Map

import Data.Graph.Clustering.Louvain.Types

data LouvainNode = LouvainNode { l_node_id :: Int
                               , l_community_id :: Int
                               } deriving (Show)

label' :: (Graph gr) => gr a b -> Edge -> Maybe b
label' gr (u, v) = lookup v (lsuc gr u)

shortest_path :: (Real b, Graph gr) => gr a b -> Node -> Node -> Maybe Path
shortest_path graph node_1 node_2 = sp node_1 node_2 graph

mkGraph' :: [LEdge b] -> Gr () b
mkGraph' es = mkGraph ns es
    where
        ns :: [LNode ()]
        ns = zip [1.. (fromIntegral . length) ns'] (repeat ())
            where ns' = nub $ concat (Protolude.map edge2nodes es)

edge2nodes :: LEdge b -> [Node]
edge2nodes (a,b,_) = [a,b]

map2graph :: Map.Map (Node, Node) b -> Gr () b
map2graph m = mkGraph' $ map (\((n1,n2), w) -> (n1,n2,w)) $ Map.toList m


mkFGraph :: [LNode a] -> [LEdge Double] -> FGraph a ()
mkFGraph ns es = toFGraph $ mkGraph ns es


mkFGraph' :: [LEdge Double] -> FGraph () ()
mkFGraph' = toFGraph . mkGraph'


-- | Utility function to remap Gr () Double into FGraph () ()
toFGraph :: forall a. Gr a Double -> FGraph a ()
toFGraph gr = gmap remap gr
  where
    remap :: Context a Double -> Context a (Weight, ())
    remap (p, v, l, s) = (p', v, l, s')
      where
      edgeMap (w, n) = ((Weight w, ()), n)
      p' = map edgeMap p
      s' = map edgeMap s

-- | Fixed point with at most n iterations
-- 'Int' argument is the maximal number of iterations to make
-- 'a -> a' is the iterator function
-- 'a -> Bool' is the condition checking function ('True' continues looping, 'False' breaks it)
-- 'a' is the initial value
fixPt :: Int -> (a -> a) -> (a -> Bool) -> a -> a
fixPt 0 iterator _ init = iterator init
fixPt n iterator cond init =
  if cond next
     then fixPt (n - 1) iterator cond init
     else next
  where
    next = iterator init
