#!/usr/bin/env python3
#
# In ipython:
# %load "src/Data/Graph/Clustering/example.py"

import numpy as np
import numpy.linalg as npl
import pandas as pd
import random
from sklearn.datasets.samples_generator import make_blobs

SAME_CLUSTER_PROB = 0.8
DIFFERENT_CLUSTER_PROB = 0.1

X, y = make_blobs(n_samples=300, centers=4, cluster_std=0.60, random_state=0)
# Norm with be the edge weight
norms = [npl.norm(xx) for xx in X]

# Our graph will consist of pt (0.0, 0.0) with edges to all nodes in X
#node0 = (0, np.array([0.0, 0.0]))
graph = {
    'nodes': [],
    'edges': [],
}

for i, (node, norm, cluster) in enumerate(zip(X, norms, y)):
    graph['nodes'].append((i, node))
    for j, (node2, cluster2) in enumerate(zip(X, y)):
        # If same cluster -- higher probability of edge creation
        # If different cluster -- low probability of an edge
        prob = DIFFERENT_CLUSTER_PROB
        if cluster == cluster2:
            prob = SAME_CLUSTER_PROB
        if random.random() < prob:
            graph['edges'].append((i, j, norm))
        # If no edge, try backwards one
        elif random.random() < prob:
            graph['edges'].append((j, i, norm))

# Custom graph file format:
# <edge source> <edge target> <edge weight>
with open('clustered.graph', 'w') as f:
    for edge in graph['edges']:
        f.write('{} {} {}\n'.format(*edge))
