-- | Specific FGL needed functions

module Data.Graph.FGL where

import Protolude
import Data.Graph.Inductive

-- | Get label of an 'LNode'
llab :: LNode a -> a
llab (_, a) = a

-- | Node labels
lnodes :: (DynGraph gr) => gr a b -> [a]
lnodes gr = mapMaybe (lab gr) $ nodes gr

-- | Given a 'DynGraph', replace a given 'LNode a' with new label (of type 'a')
replaceLNode ::  (DynGraph gr) => gr a b -> LNode a -> gr a b
replaceLNode gr (n, ln) = gmap replacer gr
  where
    replacer (p, v, l, s) =
      if v == n then (p, v, ln, s) else (p, v, l, s)

-- | Find LNode of a node (i.e. a node with label)
lnode :: (Graph gr) => gr a b -> Node -> Maybe (LNode a)
lnode gr n = case lab gr n of
  Nothing -> Nothing
  Just l  -> Just (n, l)

-- | Fold over graph definitions: type and function

-- | Fold over graph type
type CFunFold a b c = Context a b -> c -> c

-- | Fold over graph function
xdfsFoldWith :: (Graph gr)
    => CFun     a b [Node]
    -> CFunFold a b c
    -> c
    -> [Node]
    -> gr a b
    -> c
xdfsFoldWith _ _ acc [] _            = acc
xdfsFoldWith _ _ acc _ g | isEmpty g = acc
xdfsFoldWith d f acc (v:vs) g =
  case match v g of
    (Just c, g')  -> xdfsFoldWith d f (f c acc) (d c++vs) g'
    (Nothing, g') -> xdfsFoldWith d f acc vs g'
